package Ex8;

our $VERSION = 0.01;

1;

=pod

=head1 EXAMPLE

 use strict;
 use warnings;

 # Print.
 print "Foo.\n";

=begin text

   +-------------+
   |  foo        |
   |        bar  |
   +-------------+

 ^^^^ Figure 1. ^^^^

=end html

=cut
